/*
    BOB'S EYES PRO version 2
    Copyright (C) 1991-1993  Roberto Avanzi

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

	Author's Address:
		Roberto Avanzi
		via Luigi Balzan 12
		45100 Rovigo (RO)
		Italy (EEC)
	
	Internet E-Mail Address: (at least for 1992)
		gandalf@sabrina.dei.unipd.it
		caraccio@libertad.physics.nyu.edu (alternate one)
*/

#include	<SANE.h>
#include	"BobsEyesDefsStructs.h"
#include	"BobsEyesProtos.h"
#include	"ExternGlobals.h"

/*	ricalcola le dimensioni di una finestra e quelle delle pupille quando
	la finestra ha cambiato dimensione oppure e' stata appena aperta      */
/*	this procedure computes the sizes of all the parts of a window
	when the user changes its size ar a new one is opened. */

void CalcWindowSizes ()
{
	Rect		winRect, globeRect, eyeRect;
	register	Point	rectDim;
	
	winRect = theWindow->portRect;
	rectDim.h = winRect.right - winRect.left;
	rectDim.v = winRect.bottom - winRect.top;
	((EyesPeek) theWindow)->rectDim = rectDim;

	SETRECT ( & eyeRect , 0, 0, (TIMES15(rectDim.h)) >> 5, rectDim.v);

#if 1
	if ( rectDim.h < 50 )
	{
		globeRect.left = eyeRect.left + 1;
		globeRect.right = eyeRect.right - 1;
	} else {
		globeRect.left = rectDim.h >> 5;
		globeRect.right = eyeRect.right + eyeRect.left - globeRect.left;
	}
	if ( rectDim.v < 25 )
	{
		globeRect.top = eyeRect.top + 1;
		globeRect.bottom = eyeRect.bottom - 1;
	} else {
		globeRect.top = rectDim.v >> 4;
		globeRect.bottom = eyeRect.bottom + eyeRect.top - globeRect.top;
	}
#else
	{
		/*	I have discarded this one because it is not appealing. */
		short	hIndent, vIndent;
	
		if ( ( rectDim.h < 50 ) || ( rectDim.v < 25 ) )
			hIndent = vIndent = 1;
		else
		{
			hIndent = rectDim.h >> 5;
			vIndent = rectDim.v >> 4;
			if (hIndent > vIndent) hIndent = vIndent; else vIndent = hIndent;
		}
		globeRect.left = eyeRect.left + hIndent;
		globeRect.right = eyeRect.right - hIndent;
		globeRect.top = eyeRect.top + vIndent;
		globeRect.bottom = eyeRect.bottom - vIndent;
	}
#endif	

	((EyesPeek) theWindow)->pupilRay.h = (rectDim.h * 13 ) >> 7;
	((EyesPeek) theWindow)->pupilRay.v = (TIMES7(rectDim.v)) >> 5;

	if ( rectDim.h >= 50 )
		((EyesPeek) theWindow)->offset = rectDim.h - eyeRect.right;
	else
		((EyesPeek) theWindow)->offset = rectDim.h - globeRect.right - 1;

	((EyesPeek) theWindow)->globeRect = globeRect;
	((EyesPeek) theWindow)->eyeRect = eyeRect;

}  /* CalcWindowSizes */

/*  Calcola la posizione delle pupille di una certa finestra
	secondo la posizione del puntatore sullo schermo	*/
void CalcPupilPos( )
{
	((EyesPeek) theWindow)->inPupilla = 0;
	CalcPupilPosOne( &((EyesPeek) theWindow)->left, false);
	CalcPupilPosOne( &((EyesPeek) theWindow)->right, true);
}  /* CalcPupilPos */

/*	determina la posizione di UNA sola pupilla */
void CalcPupilPosOne ( PCouplePtr couple, Boolean xOffset)
{	
	Point	rectDim,point;
	Point	dist;
	Point	pupilCentre;
	Point	pupilRay, eyeCentre;
	long	integerDist, pupilDist;
	float	fTemp;
	
	rectDim = ((EyesPeek) theWindow)->rectDim;
	pupilRay = ((EyesPeek) theWindow)->pupilRay;

	eyeCentre.h = ( ( (rectDim.h << 4) - rectDim.h ) >> 6 );
	if (xOffset) eyeCentre.h += ((EyesPeek) theWindow)->offset;

	eyeCentre.v = (rectDim.v >> 1);
	
	point = gMouse;
	GlobalToLocal (&point);
	dist.h = point.h - eyeCentre.h;
	dist.v = point.v - eyeCentre.v;
		/* semplice geometria analitica : intersezione tra retta ed ellisse */
	integerDist = (long)dist.h*(long)dist.h + (long)dist.v*(long)dist.v;
	if (integerDist == 0) {
		pupilCentre.h = pupilCentre.v = 0;
		((EyesPeek) theWindow)->inPupilla = 1 + xOffset;
	} else {
		if ( dist.h == 0 ) {
			pupilCentre.h = 0;
			pupilCentre.v = pupilRay.v - 1;	// 1.3.3 sottraggo 1 per bellezza
		} else if ( dist.v == 0 ) {
			pupilCentre.h = pupilRay.h - 1;	// 1.3.3 sottraggo 1 per bellezza
			pupilCentre.v = 0;
		} else {
			fTemp = ((float)pupilRay.h*(float)dist.v)/((float)pupilRay.v*(float)dist.h);
					/* fTemp non deve essere mai zero o 1/fTemp da' errore */
			fTemp = fTemp*fTemp + .001;
			pupilCentre.h = (short) ( ((float)pupilRay.h)/sqrt(fTemp + 1));
			pupilCentre.v = (short) ( ((float)pupilRay.v)/sqrt(1/fTemp + 1));
		}
		pupilDist = ((long)pupilCentre.h*(long)pupilCentre.h
										+ (long)pupilCentre.v*(long)pupilCentre.v);
		if ( integerDist <= pupilDist )  {
			pupilCentre = dist;
		} else {
			if (dist.h < 0) pupilCentre.h = -pupilCentre.h;
			if (dist.v < 0) pupilCentre.v = -pupilCentre.v;
		}
		((EyesPeek) theWindow)->inPupilla = 1 + ( (point.h > (rectDim.h >> 1)) ? 1 : 0 );

	}
	pupilCentre.h += eyeCentre.h;
	pupilCentre.v += eyeCentre.v;
	couple->new = pupilCentre;
}    /* CalcPupilPosOne */
