/*
    BOB'S EYES PRO version 2
    Copyright (C) 1991-1992  Roberto Avanzi

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

	Author's Address:
		Roberto Avanzi
		via Luigi Balzan 12
		45100 Rovigo (RO)
		Italy (EEC)
	
	Internet E-Mail Address: (at least for 1992)
		gandalf@sabrina.dei.unipd.it
		caraccio@libertad.physics.nyu.edu (alternate one)
*/

#include	<Files.h>
#include	<GestaltEqu.h>
#include	<Folders.h>

#include	"BobsEyesDefsStructs.h"
#include	"BobsEyesProtos.h"
#include	"ExternGlobals.h"

extern	short	timesToMaxMem,gTicksToWait;
char	prefsFileName[32] = "\pBob's Eyes Pro Preferences";
extern	short	prefsFileRefNum;
extern short  appRefNum;

#define	min(A,B) (A>B)?B:A
#define	max(A,B) (A>B)?A:B

OSErr FindPrefsFolder(short *foundVRefNum, long *foundDirID);
OSErr CreatePrefsFile(short * refNum );
OSErr FindPrefsFile(short * refNum);
OSErr SaveDummySettings(void);

//	I learnt how to do it from John Norstad's disinfectant 2.4 sample source code
//	and an Apple code snippet ( FindSysFolder ) , just modified to point at
//	the preferences folder and not to the system folder.
OSErr FindPrefsFile(short * refNum)
{
	WDPBRec	pBlockHFS;	// param block for PBHSetVol call
	OSErr	error;
	long	theDirID;
	short	theVRefNum;

	//	find the preferences folder under sys7++ or the system folder
	if ( error = FindPrefsFolder(&theVRefNum,&theDirID) ) return error;
	
	//	sets the default directory to the Preferences Folder or the System Folder
	pBlockHFS.ioNamePtr = 0L;
	pBlockHFS.ioVRefNum = theVRefNum;
	pBlockHFS.ioWDDirID = theDirID;
	error = PBHSetVol((WDPBPtr)&pBlockHFS, false);

	//	try to open the Preferences File and return the error if any
	*refNum = OpenRFPerm(&prefsFileName, 0, fsRdWrPerm);
	if ( *refNum == -1 )
		return CreatePrefsFile ( refNum );

	return 0;
}

OSErr CreatePrefsFile(short * refNum )
{
	OSErr	error, error2;
	FInfo	fndrInfo;	// Finder info

	error = 0;
	CreateResFile(&prefsFileName);
	if (!(error = ResError())) {
		if (!(error = GetFInfo(&prefsFileName, 0, &fndrInfo))) {
			fndrInfo.fdType = 'PREF';
			fndrInfo.fdCreator = 'Moce';
			if (!(error = SetFInfo(&prefsFileName, 0, &fndrInfo))) {
				*refNum = OpenRFPerm(&prefsFileName, 0, fsRdWrPerm);
				if ( error2 = SaveDummySettings() ) return error2;
			}
			error = *refNum == -1;
		}
	}
	if ( error != noErr) *refNum = -1;
	return error;
}

OSErr SaveDummySettings ()
{
	Rect	newRect;
	eYeSWinPeek theWinData;
	RGBColor	*color;

	eYeS_H = (eYeSHandle) NewHandleClear(sizeof(eYeSResource));
	if ( !eYeS_H ) return -1;
	
	(**eYeS_H).valore = gSoLowToSleep;
	(**eYeS_H).quante = 1;
	
	SETRECT ( &newRect,36,48,186,168);
	(theWinData = &((**eYeS_H).eYeSWin[0]))->rect = newRect;

	color = & theWinData->bordoColor;
	color->red = 0x0000;	color->green = 0x0000;	color->blue = 0xD400;
	color = & theWinData->globoColor;
	color->red = 0xFC00;	color->green = 0xF37D;	color->blue = 0x052F;
	color = & theWinData->pupillaColor;
	color->red = 0xDD6B;	color->green = 0x08C2;	color->blue = 0x06A2;

	UseResFile(prefsFileRefNum);
	AddResource(eYeS_H, 'eYeS', 128, &"\pBob's Eyes Pro Settings");
	HNoPurge(eYeS_H); 
	WriteResource(eYeS_H);
	return 0;
}

void DoSaveSettings()
{
	WindowPtr	window, nextWindow;
	register	short	i;
	eYeSWinPeek theWinData;
	Boolean		isNew;
	Handle		handle;
	nextWindow = 0L;

beginning:
	if ( prefsFileRefNum == -1 )
		if ( FindPrefsFile(&prefsFileRefNum) != noErr ) {
			(void) Alert(rPrefsFileProblem, 0L);
			return;
		}
	//	now we can assume the prefs file has been opened
	//	let us check if the eYeS resource is in a good state
	if ( handle = GetResource ('eYeS', 128)) {
		if ( GetHandleSize(handle) != sizeof(eYeSResource) ) {
			(void) Alert(rPrefsFileDamaged, 0L);
			prefsFileRefNum = -1;
			FSDelete(&prefsFileName, 0);
			goto beginning;		// the file was found, but turned out to have been damaged
		}
	}

	UseResFile(prefsFileRefNum);
	//	now eYeS_H is a handle to a valid eYeSResource
	(**eYeS_H).valore = gSoLowToSleep;
	(**eYeS_H).quante = gNumEyeballWindows;

	HLock(eYeS_H);
	if (window = FrontWindow() ) {
		for (i = 0; i <  gNumEyeballWindows; ) {
			if ( IsAppWindow(window) ) { // v 1.1  for those working without multiFinder
										 //	and willing to use this app to run DA's ....	
				(theWinData = &((**eYeS_H).eYeSWin[i]))->rect = window->portRect;
				SetPort(window);
				LocalToGlobal(& topLeft(theWinData->rect));
				LocalToGlobal(& botRight(theWinData->rect));
				theWinData->colori = ((EyesPeek) window)->colori;
				i++;	//
			}
			window = NextWindow(window);
		}
	}
	HUnlock(eYeS_H);
	//	make the changes permanent
	ChangedResource(eYeS_H);
	WriteResource(eYeS_H);
}

//	from an Apple code snippet, modified to find the Preferences Folder under
//	sys 7 or the System Folder under sys 6, and uses the already determined
//	SysEnvRec record

#define BTstQ(arg, bitnbr)		(arg & (1 << bitnbr))
OSErr	FindPrefsFolder(short *foundVRefNum, long *foundDirID)
{
  	long			gesResponse;
  	WDPBRec			myWDPB;
  	unsigned char	volName[34];
  	OSErr			err = noErr;
  	
   	*foundVRefNum = 0;
  	*foundDirID = 0;
  	if (!Gestalt (gestaltFindFolderAttr, &gesResponse) &&
  		BTstQ (gesResponse, gestaltFindFolderPresent)) {	// Does Folder Manager exist?
  			err = FindFolder (kOnSystemDisk, kPreferencesFolderType, kCreateFolder, 
  								foundVRefNum, foundDirID);
  	} else {
  		// Gestalt can't give us the answer, so we resort to what SysEnvirons
  		//	has already determined
		myWDPB.ioVRefNum = theWorld.sysVRefNum;
		volName[0] = '\000';					// Zero volume name
		myWDPB.ioNamePtr = volName;
		myWDPB.ioWDIndex = 0;
		myWDPB.ioWDProcID = 0;
		if (!(err = PBGetWDInfo (&myWDPB, 0))) {
			*foundVRefNum = myWDPB.ioWDVRefNum;
			*foundDirID = myWDPB.ioWDDirID;
  		}
  	}
  	return (err);
}
